@@ -862,6 +862,9 @@ gnome_font_kern (const GnomeFont *font, int glyph1, int glyph2)
   int ktabsize;
   const GnomeFontKernPair *ktab;
   int j;
+
+  g_return_val_if_fail (font != NULL, 0.0);
+  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
     
   ktabsize = font->fontmap_entry->num_kerns;
   ktab = font->fontmap_entry->kerns;
@@ -880,6 +883,8 @@ gnome_font_unsized_kern (const GnomeFontUnsized *font, int glyph1, int glyph2)
   int ktabsize;
   const GnomeFontKernPair *ktab;
   int j;
+
+  g_return_val_if_fail (font != NULL, 0);
     
   ktabsize = font->num_kerns;
   ktab = font->kerns;
@@ -936,6 +941,8 @@ gnome_font_new (const char *name, double size)
   int n_fonts = klass->n_fonts;
   int i;
 
+  g_return_val_if_fail (name != NULL, NULL);
+
   for (i = 0; i < n_fonts; i++)
     if (!strcmp (name, fontmap[i].font_name))
       break;
@@ -1036,6 +1043,9 @@ gnome_font_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 double
 gnome_font_get_ascender (const GnomeFont *font)
 {
+  g_return_val_if_fail (font != NULL, 0.0);
+  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0); 
+
   return font->fontmap_entry->ascender * font->scale;
 }
 
@@ -1052,6 +1062,9 @@ gnome_font_get_ascender (const GnomeFont *font)
 double
 gnome_font_get_descender (const GnomeFont *font)
 {
+  g_return_val_if_fail (font != NULL, 0.0);
+  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0); 
+
   return font->fontmap_entry->descender * font->scale;
 }
 
@@ -1068,6 +1081,9 @@ gnome_font_get_descender (const GnomeFont *font)
 double
 gnome_font_get_underline_position (const GnomeFont *font)
 {
+  g_return_val_if_fail (font != NULL, 0.0);
+  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0); 
+
   return font->fontmap_entry->underline_position * font->scale;
 }
 
@@ -1084,6 +1100,9 @@ gnome_font_get_underline_position (const GnomeFont *font)
 double
 gnome_font_get_underline_thickness (const GnomeFont *font)
 {
+  g_return_val_if_fail (font != NULL, 0.0);
+  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0); 
+
   return font->fontmap_entry->underline_thickness * font->scale;
 }
 
@@ -1100,6 +1119,8 @@ gnome_font_unsized_closest (const char *family_name,
   int i;
   int best, best_dist, dist;
 
+  g_return_val_if_fail (family_name != NULL, NULL);
+
   /* This should be reimplemented to use the gnome_font_family_hash. */
   best = -1;
   best_dist = 1000000;
@@ -1147,6 +1168,8 @@ gnome_font_new_closest (const char *family_name,
   int i;
   int best, best_dist, dist;
 
+  g_return_val_if_fail (family_name != NULL, NULL);
+
   best = -1;
   best_dist = 1000000;
   for (i = 0; i < n_fonts; i++)
@@ -1211,6 +1234,7 @@ const char *
 gnome_font_get_name (const GnomeFont *font)
 {
   g_return_val_if_fail (font != NULL, NULL);
+  g_return_val_if_fail (GNOME_IS_FONT (font), NULL);
 
   return font->fontmap_entry->font_name;
 }
@@ -1222,6 +1246,7 @@ gnome_font_get_glyph_name (const GnomeFont *font)
   const GnomeFontMap *fontmap_entry;
 
   g_return_val_if_fail (font != NULL, NULL);
+  g_return_val_if_fail (GNOME_IS_FONT (font), NULL);
 
   fontmap_entry = font->fontmap_entry;
 
@@ -1381,18 +1406,25 @@ gnome_font_unsized_get_pfa (const GnomeFontUnsized *font)
 char *
 gnome_font_get_full_name (const GnomeFont *font)
 {
+  g_return_val_if_fail (font != NULL, NULL);
+  g_return_val_if_fail (GNOME_IS_FONT (font), NULL);
   return g_strdup_printf("%s %f", gnome_font_get_name(font), font->size);
 }
 
 char *
 gnome_font_get_pfa (const GnomeFont *font)
 {
+  g_return_val_if_fail (font != NULL, NULL);
+  g_return_val_if_fail (GNOME_IS_FONT (font), NULL);
   return gnome_font_unsized_get_pfa (font->fontmap_entry);
 }
 
 double
 gnome_font_get_width (const GnomeFont *font, int ch)
 {
+  g_return_val_if_fail (font != NULL, 0.0);
+  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
+
   if (ch < 0 || ch >= 256)
     return 0.0;
   return font->fontmap_entry->widths[ch] * font->scale;
@@ -1402,6 +1434,8 @@ gnome_font_get_width (const GnomeFont *font, int ch)
 int
 gnome_font_unsized_get_width (const GnomeFontUnsized *font, int ch)
 {
+  g_return_val_if_fail (font != NULL, 0);
+
   if (ch < 0 || ch >= 256)
     return 0;
   return font->widths[ch];
@@ -1421,11 +1455,18 @@ gnome_font_unsized_get_width (const GnomeFontUnsized *font, int ch)
 double
 gnome_font_get_width_string (const GnomeFont *font, const char *s)
 {
-  unsigned char *us = (unsigned char *)s;
-  int *widths = font->fontmap_entry->widths;
+  unsigned char *us = NULL;
+  int *widths = NULL;
   double width;
   unsigned char c;
 
+  g_return_val_if_fail (font != NULL, 0.0);
+  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
+  g_return_val_if_fail (s != NULL, 0.0);
+
+  us = (unsigned char *)s;
+  widths = font->fontmap_entry->widths;
+
   width = 0;
   while ((c = *(us++)) != 0)
     width += widths[c];
@@ -1448,11 +1489,20 @@ gnome_font_get_width_string (const GnomeFont *font, const char *s)
 double
 gnome_font_get_width_string_n (const GnomeFont *font, const char *s, int n)
 {
-  unsigned char *us = (unsigned char *)s;
-  int const * const widths = font->fontmap_entry->widths;
   double width = 0.0;
-  while (--n >= 0)
-    width += widths[*(us++) ];
+
+  g_return_val_if_fail (font != NULL, 0.0);
+  g_return_val_if_fail (GNOME_IS_FONT (font), 0.0);
+  g_return_val_if_fail (s != NULL, 0.0);
+
+  {
+    int const * const widths = font->fontmap_entry->widths;
+    unsigned char *us  = (unsigned char *)s;
+
+    while (--n >= 0)
+      width += widths[*(us++) ];
+  }
+
   return width * font->scale;
 }
 
@@ -1464,6 +1514,8 @@ gnome_font_unsized_get_glyph (const GnomeFontUnsized *font, int unicode)
   int page;
   int *cov_page;
 
+  g_return_val_if_fail (font != NULL, -1);
+
   page = unicode >> 8;
   page -= font->first_cov_page;
   if (page < 0 || page >= font->num_cov_pages)
@@ -1480,6 +1532,9 @@ gnome_font_unsized_get_glyph (const GnomeFontUnsized *font, int unicode)
 int
 gnome_font_get_glyph (const GnomeFont *font, int unicode)
 {
+  g_return_val_if_fail (font != NULL, -1);
+  g_return_val_if_fail (GNOME_IS_FONT (font), -1);
+
   return gnome_font_unsized_get_glyph (font->fontmap_entry, unicode);
 }
 
@@ -1502,7 +1557,12 @@ gnome_font_list (GnomeFontClass *class)
 {
   GList *list;
   int i;
-  GnomeFontMap *fontmap = class->fontmap;
+  GnomeFontMap *fontmap;
+
+  g_return_val_if_fail (class != NULL, NULL);
+  g_return_val_if_fail (GNOME_IS_FONT_CLASS (class), NULL);
+
+  fontmap = class->fontmap;
 
   list = NULL;
 
@@ -1530,7 +1590,12 @@ gnome_font_family_list (GnomeFontClass *class)
   GHashTable *hash;
   int i;
   GList *the_family;
-  GnomeFontMap *fontmap = class->fontmap;
+  GnomeFontMap *fontmap;
+
+  g_return_val_if_fail (class != NULL, NULL);
+  g_return_val_if_fail (GNOME_IS_FONT_CLASS (class), NULL);
+
+  fontmap = class->fontmap;
 
   if (gnome_font_family_the_list != NULL)
     return gnome_font_family_the_list;
@@ -1869,7 +1934,7 @@ static GdkFont *find_best_x_font (const char *unsized_name,
   /* sample string used during the search for a good size */
   /*char *test_string = "abcdefghijklmnopqrstuvwxyz";*/
   /*char *test_string = "snider";*/
-  const char *test_string = "oooowwww";
+  /* const char *test_string = "oooowwww"; */
 #if 0
   int length = strlen (test_string);
 
