@@ -71,7 +71,7 @@ gnome_print_dialog_finalize (GtkObject *object)
 {
 	GnomePrintDialog *gpd = GNOME_PRINT_DIALOG(object);
 
-	g_free(gpd->private);
+	g_free(gpd->priv);
 
 	/* all widgets should be free'd by their containers? */
 
@@ -97,7 +97,7 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG (gpd)->vbox), GTK_WIDGET (gpd->printer), FALSE, FALSE, 3);
 	gtk_widget_show(gpd->printer);
 	gpd->range_type = GNOME_PRINT_RANGETYPE_NONE;
-	gpd->private = g_malloc0(sizeof(Private));
+	gpd->priv = g_malloc0(sizeof(Private));
 }
 
 #define PAD 4
@@ -235,7 +235,7 @@ gnome_print_dialog_construct_range_any(GnomePrintDialog *gpd, int flags, GtkWidg
 	g_return_if_fail(GNOME_IS_PRINT_DIALOG(gpd));
 	g_return_if_fail(gpd->range_container != NULL);
 
-	p = gpd->private;
+	p = gpd->priv;
 
 	row=0;
 	if (flags&GNOME_PRINT_RANGE_SELECTION)
@@ -324,7 +324,7 @@ gnome_print_dialog_construct_range_date(GnomePrintDialog *gpd, int flags, time_t
 	/* cannot print all dates ... */
 	flags &= ~GNOME_PRINT_RANGE_ALL;
 
-	p = gpd->private;
+	p = gpd->priv;
 
 	if (flags&GNOME_PRINT_RANGE_RANGE) {
 		int dateflags=0;
@@ -381,7 +381,7 @@ gnome_print_dialog_construct_range_page(GnomePrintDialog *gpd, int flags, int st
 	g_return_if_fail(GNOME_IS_PRINT_DIALOG(gpd));
 	g_return_if_fail(gpd->range_container != NULL);
 
-	p = gpd->private;
+	p = gpd->priv;
 
 	if (flags&GNOME_PRINT_RANGE_RANGE) {
 		hbox = gtk_hbox_new(FALSE, 3);
@@ -422,7 +422,7 @@ gnome_print_dialog_construct_range_page(GnomePrintDialog *gpd, int flags, int st
  * 
  * Return value: A bitmask with one option set.
  **/
-int
+GnomePrintRangeType
 gnome_print_dialog_get_range(GnomePrintDialog *gpd)
 {
 	Private *p;
@@ -431,7 +431,7 @@ gnome_print_dialog_get_range(GnomePrintDialog *gpd)
 	g_return_val_if_fail(GNOME_IS_PRINT_DIALOG(gpd), 0);
 	g_return_val_if_fail(gpd->range_container != NULL, 0);
 
-	p = gpd->private;
+	p = gpd->priv;
 
 	if (GTK_TOGGLE_BUTTON(p->current)->active)
 		return GNOME_PRINT_RANGE_CURRENT;
@@ -467,7 +467,7 @@ gnome_print_dialog_get_range_page(GnomePrintDialog *gpd, int *start, int *end)
 	g_return_val_if_fail(gpd->range_container != NULL, 0);
 	/*	g_return_val_if_fail(gpd->range_type != GNOME_PRINT_RANGETYPE_PAGES, 0);*/
 
-	p = gpd->private;
+	p = gpd->priv;
 
 	mask = gnome_print_dialog_get_range(gpd);
 	if (mask & GNOME_PRINT_RANGE_RANGE) {
@@ -501,7 +501,7 @@ gnome_print_dialog_get_range_date(GnomePrintDialog *gpd, time_t *start, time_t *
 	g_return_val_if_fail(gpd->range_container != NULL, 0);
 	g_return_val_if_fail(gpd->range_type != GNOME_PRINT_RANGETYPE_DATES, 0);
 
-	p = gpd->private;
+	p = gpd->priv;
 
 	mask = gnome_print_dialog_get_range(gpd);
 	if (mask & GNOME_PRINT_RANGE_RANGE) {
