@@ -70,12 +70,18 @@ struct _Private {
 	/*
 	 * Used for dragging the sheet
 	 */
-	int                 drag_anchor_x, drag_anchor_y;
-	int                 drag_ofs_x, drag_ofs_y;
+	int                  drag_anchor_x, drag_anchor_y;
+	int                  drag_ofs_x, drag_ofs_y;
+
+	gboolean             landscape;
+	/* Dimensions of the paper.  In landscape mode, width is probably
+	   bigger than height since landscape is taken into account.  */
+	gdouble              width, height;
 };
 
 static void preview_zoom_fit_cmd(GtkToggleButton *t, GnomePrintMasterPreview *pmp);
 
+
 /*
  * Padding in points around the simulated page
  */
@@ -86,7 +92,7 @@ render_page (GnomePrintMasterPreview *pmp, int page)
 {
 	const GnomePaper *paper;
 	const char *paper_name;
-	struct _Private *pp = pmp->private;
+	struct _Private *pp = pmp->priv;
 
 	gtk_object_destroy (GTK_OBJECT (pp->output));
 	pp->output = NULL;
@@ -105,8 +111,18 @@ render_page (GnomePrintMasterPreview *pmp, int page)
 		pmp->canvas,
 		0 - PAGE_PAD,
 		0 - PAGE_PAD,
-		gnome_paper_pswidth (paper) + PAGE_PAD,
-		gnome_paper_psheight (paper) + PAGE_PAD);
+		pp->width + PAGE_PAD,
+		pp->height + PAGE_PAD);
+
+	if (pp->landscape) {
+		double affine [6];
+
+		art_affine_translate (affine, 0, pp->width);
+		gnome_print_concat (GNOME_PRINT_CONTEXT (pp->output), affine);
+
+		art_affine_rotate (affine, -90.0);
+		gnome_print_concat (GNOME_PRINT_CONTEXT (pp->output), affine);
+	}
 
 	gnome_print_meta_render_from_object_page (GNOME_PRINT_CONTEXT(pp->output),
 						  GNOME_PRINT_META (gnome_print_master_get_context(pmp->master)),
@@ -116,14 +132,14 @@ render_page (GnomePrintMasterPreview *pmp, int page)
 static void
 goto_page (GnomePrintMasterPreview *pmp, int page)
 {
-	char text[16];
+	char text[4 * sizeof (int)];
 
 	if (page == pmp->current_page)
 		return;
 
 	pmp->current_page = page;
 
-	sprintf(text, "%d", page+1);
+	sprintf (text, "%d", page + 1);
 	gtk_entry_set_text (GTK_ENTRY (pmp->page_entry), text);
 
 	render_page (pmp, page);
@@ -168,7 +184,7 @@ static gint
 preview_canvas_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	GnomePrintMasterPreview *pmp = data;
-	struct _Private *pp = pmp->private;
+	struct _Private *pp = pmp->priv;
 	int retval;
 
 	retval = FALSE;
@@ -207,7 +223,7 @@ preview_canvas_button_press (GtkWidget *widget, GdkEventButton *event, gpointer
 static void
 drag_to (GnomePrintMasterPreview *pmp, int x, int y)
 {
-	struct _Private *pp = pmp->private;
+	struct _Private *pp = pmp->priv;
 	int dx, dy;
 
 	dx = pp->drag_anchor_x - x;
@@ -229,7 +245,7 @@ static gint
 preview_canvas_button_release (GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	GnomePrintMasterPreview *pmp = data;
-	struct _Private *pp = pmp->private;
+	struct _Private *pp = pmp->priv;
 	int retval;
 
 	retval = FALSE;
@@ -258,7 +274,7 @@ static gint
 preview_canvas_motion (GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
 	GnomePrintMasterPreview *pmp = data;
-	struct _Private *pp = pmp->private;
+	struct _Private *pp = pmp->priv;
 	int retval;
 	gint x, y;
 	GdkModifierType mods;
@@ -344,39 +360,42 @@ preview_zoom_out_cmd (GtkToggleButton *t, GnomePrintMasterPreview *pmp)
 }
 
 static void
-preview_zoom_fit_cmd(GtkToggleButton *t, GnomePrintMasterPreview *pmp)
+preview_zoom_fit_cmd (GtkToggleButton *t, GnomePrintMasterPreview *pmp)
 {
 	double zoomx, zoomy;
-	int width = GTK_WIDGET(pmp->canvas)->allocation.width;
-	int height = GTK_WIDGET(pmp->canvas)->allocation.height;
+	struct _Private *pp = pmp->priv;
+	int width = GTK_WIDGET (pmp->canvas)->allocation.width;
+	int height = GTK_WIDGET (pmp->canvas)->allocation.height;
 	
-	zoomx = width/(gnome_paper_pswidth (pmp->master->paper)+5.0+PAGE_PAD);
-	zoomy = height/(gnome_paper_psheight (pmp->master->paper)+5.0+PAGE_PAD);
+	zoomx = width / (pp->width + 5.0 + PAGE_PAD);
+	zoomy = height / (pp->height + 5.0 + PAGE_PAD);
 	
-	if (zoomy<zoomx)
+	if (zoomy < zoomx)
 		zoomx = zoomy;
 	gnome_canvas_set_pixels_per_unit (pmp->canvas, zoomx);
 }
 
 static void
-preview_zoom_fit_wide_cmd(GtkToggleButton *t, GnomePrintMasterPreview *pmp)
+preview_zoom_fit_wide_cmd (GtkToggleButton *t, GnomePrintMasterPreview *pmp)
 {
 	double zoom;
-	int width = GTK_WIDGET(pmp->canvas)->allocation.width;
-	
+	struct _Private *pp = pmp->priv;
+	int width = GTK_WIDGET (pmp->canvas)->allocation.width;
+
 	/* include paper decoration */
-	zoom = width/(gnome_paper_pswidth (pmp->master->paper)+5.0+PAGE_PAD);
+	zoom = width / (pp->width + 5.0 + PAGE_PAD);
 	gnome_canvas_set_pixels_per_unit (pmp->canvas, zoom);
 }
 
 static void
-preview_zoom_fit_tall_cmd(GtkToggleButton *t, GnomePrintMasterPreview *pmp)
+preview_zoom_fit_tall_cmd (GtkToggleButton *t, GnomePrintMasterPreview *pmp)
 {
 	double zoom;
-	int height = GTK_WIDGET(pmp->canvas)->allocation.height;
+	struct _Private *pp = pmp->priv;
+	int height = GTK_WIDGET (pmp->canvas)->allocation.height;
 	
 	/* include paper decoration */
-	zoom = height/(gnome_paper_psheight (pmp->master->paper)+5.0+PAGE_PAD);
+	zoom = height / (pp->height + 5.0 + PAGE_PAD);
 	gnome_canvas_set_pixels_per_unit (pmp->canvas, zoom);
 }
 
@@ -502,7 +521,7 @@ static void
 create_preview_canvas (GnomePrintMasterPreview *pmp)
 {
 	GtkWidget *box, *status;
-	struct _Private *pp = pmp->private;
+	struct _Private *pp = pmp->priv;
 	const GnomePaper *paper;
 	const char *paper_name;
 	GnomeCanvasItem *i;
@@ -545,8 +564,8 @@ create_preview_canvas (GnomePrintMasterPreview *pmp)
 		gnome_canvas_rect_get_type (),
 		"x1",   	 0.0,
 		"y1",   	 0.0,
-		"x2",   	 (double) gnome_paper_pswidth (paper),
-		"y2",   	 (double) gnome_paper_psheight (paper),
+		"x2",   	 (double) pp->width,
+		"y2",   	 (double) pp->height,
 		"fill_color",    "white",
 		"outline_color", "black",
 		"width_pixels",  1,
@@ -557,8 +576,8 @@ create_preview_canvas (GnomePrintMasterPreview *pmp)
 		gnome_canvas_rect_get_type (),
 		"x1",   	 3.0,
 		"y1",   	 3.0,
-		"x2",   	 (double) gnome_paper_pswidth (paper) + 3,
-		"y2",   	 (double) gnome_paper_psheight (paper) + 3,
+		"x2",   	 (double) pp->width + 3,
+		"y2",   	 (double) pp->height + 3,
 		"fill_color",    "black",
 		NULL);
 	gnome_canvas_item_lower_to_bottom (i);
@@ -566,8 +585,8 @@ create_preview_canvas (GnomePrintMasterPreview *pmp)
 		pmp->canvas,
 		0 - PAGE_PAD,
 		0 - PAGE_PAD,
-		gnome_paper_pswidth (paper) + PAGE_PAD,
-		gnome_paper_psheight (paper) + PAGE_PAD);
+		pp->width + PAGE_PAD,
+		pp->height + PAGE_PAD);
 
 	box = gtk_vbox_new (FALSE, 0);
 	status = gtk_hbox_new (FALSE, 0);
@@ -658,11 +677,19 @@ create_toplevel (GnomePrintMasterPreview *pmp)
 
 	g_return_if_fail (pmp != NULL);
 
-	pp = pmp->private;
+	pp = pmp->priv;
 
 	paper  = pmp->master->paper;
-	width  = gnome_paper_pswidth  (paper) + PAGE_PAD * 3;
-	height = gnome_paper_psheight (paper) + PAGE_PAD * 3;
+	if (pp->landscape) {
+		pp->height = gnome_paper_pswidth  (paper);
+		pp->width  = gnome_paper_psheight (paper);
+	} else {
+		pp->width  = gnome_paper_pswidth  (paper);
+		pp->height = gnome_paper_psheight (paper);
+	}
+
+	width  = pp->width + PAGE_PAD * 3;
+	height = pp->height + PAGE_PAD * 3;
 
 	if (width > gdk_screen_width () - 40)
 		width = gdk_screen_width () - 40;
@@ -670,7 +697,7 @@ create_toplevel (GnomePrintMasterPreview *pmp)
 	if (height > gdk_screen_height () - 100)
 		height = gdk_screen_height () - 100;
 
-	gtk_widget_set_usize (GTK_WIDGET(pmp), width, height);
+	gtk_widget_set_usize (GTK_WIDGET (pmp), width, height);
 	gtk_window_set_policy (GTK_WINDOW (pmp), TRUE, TRUE, FALSE);
 
 	gnome_app_create_menus_with_data (GNOME_APP(pmp), top_menu, pmp);
@@ -716,7 +743,7 @@ static void
 gnome_print_master_preview_finalize (GtkObject *object)
 {
 	GnomePrintMasterPreview *pmp = GNOME_PRINT_MASTER_PREVIEW(object);
-	struct _Private *pp = pmp->private;
+	struct _Private *pp = pmp->priv;
 
 	if (pp->output != NULL)
 		gtk_object_unref (GTK_OBJECT (pp->output));
@@ -745,7 +772,7 @@ gnome_print_master_preview_init (GnomePrintMasterPreview *pmp)
 {
 	struct _Private *pp;
 
-	pp = pmp->private = g_malloc0(sizeof(*pp));
+	pp = pmp->priv = g_malloc0(sizeof(*pp));
 	pmp->current_page = -1;
 }
 
@@ -760,22 +787,45 @@ gnome_print_master_preview_init (GnomePrintMasterPreview *pmp)
  * Return value: A newly created GnomePrintMasterPreview widget.
  **/
 GnomePrintMasterPreview *
-gnome_print_master_preview_new (GnomePrintMaster *gpm, char *title)
+gnome_print_master_preview_new (GnomePrintMaster *gpm,
+				const char *title)
+{
+	return gnome_print_master_preview_new_with_orientation
+		(gpm, title, FALSE);
+}
+
+/**
+ * gnome_print_master_preview_new:
+ * @gpm: A GnomePrintMaster which has been printed to and completed.
+ * @title: Window title for the preview window.
+ * @landscape: TRUE to rotate paper.
+ * 
+ * Create a new preview window widget.  The preview has inbuilt
+ * buttons for zooming and panning, and menu's to print.
+ * 
+ * Return value: A newly created GnomePrintMasterPreview widget.
+ **/
+GnomePrintMasterPreview *
+gnome_print_master_preview_new_with_orientation (GnomePrintMaster *gpm,
+						 const char *title,
+						 gboolean landscape)
 {
 	GnomePrintMasterPreview *pmp;
 	struct _Private *pp;
-	char text[16];
+	char text[4 * sizeof (int)];
 
 	pmp = GNOME_PRINT_MASTER_PREVIEW (gtk_type_new(gnome_print_master_preview_get_type()));
 
 	gnome_app_construct(GNOME_APP(pmp), "preview_window", title);
 
-	pp = pmp->private;
+	pp = pmp->priv;
+	pp->landscape = landscape;
+
 	pmp->master = gpm;
 	gtk_object_ref(GTK_OBJECT(gpm));
 
-	create_toplevel(pmp);
-	create_preview_canvas(pmp);
+	create_toplevel (pmp);
+	create_preview_canvas (pmp);
 
 	/* this zooms to fit, once we know how big the window actually is */
 	gtk_signal_connect(GTK_OBJECT(pmp->canvas), "realize",
