@@ -27,6 +27,8 @@
 
 #include <libgnomeprint/gnome-print-meta.h>
 
+#undef META_DEBUG
+
 static GnomePrintContextClass *parent_class = NULL;
 
 /* Note: this must continue to be the same length  as "GNOME_METAFILE-0.0" */
@@ -84,9 +86,44 @@ typedef enum {
 	GNOME_META_GRAYIMAGE,
 	GNOME_META_RGBIMAGE,
 	GNOME_META_SHOWPAGE,
+	GNOME_META_BEGINPAGE,
 	GNOME_META_TEXTLINE,
-	GNOME_META_CLOSE
+	GNOME_META_CLOSE,
 } GnomeMetaType;
+
+#ifdef META_DEBUG
+static char *meta_type_names[] = {
+	"NEWPATH",
+	"MOVETO",
+	"LINETO",
+	"CURVETO",
+	"CLOSEPATH",
+	"SETRGBCOLOR",
+	"FILL",
+	"EOFILL",
+	"SETLINEWIDTH",
+	"SETMITERLIMIT",
+	"SETLINEJOIN",
+	"SETLINECAP",
+	"SETDASH",
+	"STROKEPATH",
+	"STROKE",
+	"SETFONT",
+	"SHOW",
+	"CONCAT",
+	"SETMATRIX",
+	"GSAVE",
+	"GRESTORE",
+	"CLIP",
+	"EOCLIP",
+	"GRAYIMAGE",
+	"RGBIMAGE",
+	"SHOWPAGE",
+	"BEGINPAGE",
+	"TEXTLINE",
+	"CLOSE"
+};
+#endif
 	
 static void
 gnome_print_meta_finalize (GtkObject *object)
@@ -227,11 +264,18 @@ encode_page_header(GnomePrintMeta *pc, GnomeMetaPageHeader *mh)
 	pc->priv->last_page = pc->current;
 	encode_block(pc, sizeof(mh->signature), mh->signature);
 	encode_int_header(pc, mh->size);
+#ifdef META_DEBUG
+	g_warning ("Page header ends at 0x%x", pc->current);
+#endif
 }
 
 static int
 meta_newpath (GnomePrintContext *pc)
 {
+#ifdef META_DEBUG
+	g_warning ("Newpath starts at 0x%x",
+		   GNOME_PRINT_META (pc)->current);
+#endif
 	encode_int (pc, GNOME_META_NEWPATH);
 	return 0;
 }
@@ -239,6 +283,10 @@ meta_newpath (GnomePrintContext *pc)
 static int
 meta_moveto (GnomePrintContext *pc, double x, double y)
 {
+#ifdef META_DEBUG
+	g_warning ("MoveTo starts at 0x%x",
+		   GNOME_PRINT_META (pc)->current);
+#endif
 	encode_int (pc, GNOME_META_MOVETO);
 	encode_double (pc, x);
 	encode_double (pc, y);
@@ -248,6 +296,10 @@ meta_moveto (GnomePrintContext *pc, double x, double y)
 static int
 meta_lineto (GnomePrintContext *pc, double x, double y)
 {
+#ifdef META_DEBUG
+	g_warning ("LineTo starts at 0x%x",
+		   GNOME_PRINT_META (pc)->current);
+#endif
 	encode_int (pc, GNOME_META_LINETO);
 	encode_double (pc, x);
 	encode_double (pc, y);
@@ -272,6 +324,10 @@ meta_closepath (GnomePrintContext *pc)
 static int
 meta_setrgbcolor (GnomePrintContext *pc, double r, double g, double b)
 {
+#ifdef META_DEBUG
+	g_warning ("SetRGBColor starts at 0x%x",
+		   GNOME_PRINT_META (pc)->current);
+#endif
 	encode_int (pc, GNOME_META_SETRGBCOLOR);
 	encode_double (pc, r);
 	encode_double (pc, g);
@@ -296,6 +352,10 @@ meta_eofill (GnomePrintContext *pc)
 static int
 meta_setlinewidth (GnomePrintContext *pc, double width)
 {
+#ifdef META_DEBUG
+	g_warning ("SetLineWidth starts at 0x%x",
+		   GNOME_PRINT_META (pc)->current);
+#endif
 	encode_int (pc, GNOME_META_SETLINEWIDTH);
 	encode_double (pc, width);
 	return 0;
@@ -359,6 +419,10 @@ meta_setfont (GnomePrintContext *pc, GnomeFont *font)
 	const char *fontname = font->fontmap_entry->font_name;
 	int len = strlen (fontname);
 			  
+#ifdef META_DEBUG
+	g_warning ("SetFont starts at 0x%x",
+		   GNOME_PRINT_META (pc)->current);
+#endif
 	encode_int (pc, GNOME_META_SETFONT);
 	encode_double (pc, font->size);
 	encode_int (pc, len);
@@ -373,6 +437,10 @@ meta_show (GnomePrintContext *pc, char const *text)
 {
 	int len = strlen (text);
 	
+#ifdef META_DEBUG
+	g_warning ("Show starts at 0x%x",
+		   GNOME_PRINT_META (pc)->current);
+#endif
 	encode_int (pc, GNOME_META_SHOW);
 	encode_int (pc, len);
 	encode_block (GNOME_PRINT_META (pc), len, text);
@@ -471,6 +539,10 @@ meta_showpage (GnomePrintContext *pc)
 	};
 	GnomePrintMeta *meta = GNOME_PRINT_META(pc);
 
+#ifdef META_DEBUG
+	g_warning ("ShowPage starts at 0x%x",
+		   GNOME_PRINT_META (pc)->current);
+#endif
 	encode_int (pc, GNOME_META_SHOWPAGE);
 
 	/* assume another page is coming */
@@ -480,6 +552,17 @@ meta_showpage (GnomePrintContext *pc)
 	return 0;
 }
 
+static int
+meta_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
+{
+	int len = strlen (name_of_this_page);
+
+	encode_int (pc, GNOME_META_BEGINPAGE);
+	encode_int (pc, len);
+	encode_block (GNOME_PRINT_META (pc), len, name_of_this_page);
+	return 0;
+}
+
 static int
 meta_close (GnomePrintContext *pc)
 {
@@ -537,6 +620,7 @@ gnome_print_meta_class_init (GnomePrintMetaClass *class)
 	pc_class->rgbimage = meta_rgbimage;
 	pc_class->textline = meta_textline;
 	pc_class->showpage = meta_showpage;
+	pc_class->beginpage = meta_beginpage;
 	
 	pc_class->close = meta_close;
 }
@@ -823,6 +907,17 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 		gint32 opcode, i;
 		
 		data = decode_int (data, &opcode);
+#ifdef META_DEBUG
+		if (opcode >= 0 &&
+		    opcode < sizeof (meta_type_names)/sizeof(char *))
+			g_warning ("Meta op: '%s' starts at offset 0x%x",
+				   meta_type_names [opcode],
+				   size - (end - data) - 4);
+		else
+			g_warning ("Meta op: OOB %d starts at offset 0x%x",
+				   opcode, size - (end - data) - 4);
+#endif
+
 		switch ((GnomeMetaType) opcode){
 		case GNOME_META_NEWPATH:
 			gnome_print_newpath (dest);
@@ -926,7 +1021,7 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			g_free (name);
 			data += i;
 			
-			if (font){
+			if (font) {
 				gnome_print_setfont (dest, font);
 				gtk_object_unref (GTK_OBJECT (font));
 			}
@@ -997,12 +1092,30 @@ do_render (GnomePrintContext *dest, const char *data, int size, int pages)
 			}
 			break;
 
+		case GNOME_META_BEGINPAGE: {
+			char *buffer;
+
+			data = decode_int (data, &i);
+			buffer = g_malloc (i + 1);
+			memcpy (buffer, data, i);
+			buffer [i] = 0;
+			gnome_print_beginpage (dest, buffer);
+			g_free (buffer);
+			data += i;
+			break;
+		}
+			
 		case GNOME_META_TEXTLINE:
 			g_warning ("FIXME: Text line missing");
 			break;
 			
 		case GNOME_META_CLOSE:
 			return TRUE;
+
+		default:
+			g_warning ("Serious print meta data corruption %d",
+				   opcode);
+			break;
 		}
 	}
 	return TRUE;
@@ -1037,7 +1150,8 @@ gnome_print_meta_render (GnomePrintContext *destination, const void *meta_stream
 		return FALSE;
 	}
 
-	return do_render (destination, data + FILEHEADER_SIZE + PAGEHEADER_SIZE, header.size, -1);
+	return do_render (destination, data + PAGEHEADER_SIZE,
+			  header.size, -1);
 }
 
 /**
@@ -1095,10 +1209,11 @@ gnome_print_meta_render_page (GnomePrintContext *destination, const void *meta_s
 
 	page_data = locate_page_header(meta_stream, page);
 	if (page_data == NULL) {
-		g_warning("Trying to print a non-existant page");
+		g_warning ("Trying to print a non-existant page");
 		return FALSE;
 	}
-	return do_render (destination, page_data+PAGEHEADER_SIZE, header.size, 1);
+	return do_render (destination, page_data + PAGEHEADER_SIZE,
+			  header.size, 1);
 }
 
 /**
