@@ -682,6 +682,13 @@ gpp_showpage (GnomePrintContext *pc)
 	return 0;
 }
 
+static int
+gpp_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
+{
+	/* Nothing.  */
+	return 0;
+}
+
 static Gt1LoadedFont *
 fetch_font (GnomePrintPreviewPrivate *priv, const char *pfb_file, const char *afm_file)
 {
@@ -868,6 +875,7 @@ gpp_class_init (GnomePrintPreviewClass *class)
 	pc_class->rgbimage = gpp_rgbimage;
 	pc_class->textline = gpp_textline;
 	pc_class->showpage = gpp_showpage;
+	pc_class->beginpage = gpp_beginpage;
 	pc_class->setopacity = gpp_setopacity;
 	
 	pc_class->close = gpp_close;
