@@ -401,7 +401,7 @@ gnome_print_ps_show (GnomePrintContext *pc, char const *text)
 	}
       else if ( (unsigned char) text[i] > 143 )
         {
-          if (gnome_print_context_fprintf (pc, "%c", (unsigned char) text[i]) < 0)
+          if (gnome_print_context_fprintf (pc, "\\%o", (unsigned char) text[i]) < 0)
 	    return -1;
 	}
       else 
@@ -641,15 +641,30 @@ gnome_print_ps_textline (GnomePrintContext *pc, GnomeTextLine *line)
 static int
 gnome_print_ps_showpage (GnomePrintContext *pc)
 {
+  GnomePrintPs *ps = GNOME_PRINT_PS (pc);
+  ps->pageno++;
   return gnome_print_context_fprintf (pc,
                                       "showpage\n");
 }
 
+static int
+gnome_print_ps_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
+{
+  GnomePrintPs *ps = GNOME_PRINT_PS (pc);
+  return gnome_print_context_fprintf (pc,
+                                      "%%%%Page: %s %d\n",
+				      name_of_this_page, ps->pageno + 1);
+}
+
 static int
 gnome_print_ps_close (GnomePrintContext *pc)
 {
+  GnomePrintPs *ps = GNOME_PRINT_PS (pc);
   gnome_print_context_fprintf (pc,
-			       "%%%%EOF\n");
+			       "%%%%Trailer\n"
+			       "%%%%Pages: %d\n"
+			       "%%%%EOF\n",
+			       ps->pageno);
   return gnome_print_context_close_file (pc);
 }
 
@@ -694,6 +709,7 @@ gnome_print_ps_class_init (GnomePrintPsClass *class)
   pc_class->rgbimage = gnome_print_ps_rgbimage;
   pc_class->textline = gnome_print_ps_textline;
   pc_class->showpage = gnome_print_ps_showpage;
+  pc_class->beginpage = gnome_print_ps_beginpage;
 
   pc_class->close = gnome_print_ps_close;
 }
@@ -742,6 +758,8 @@ gnome_print_ps_init (GnomePrintPs *ps)
 {
   int i;
 
+  ps->pageno = 0;
+
   ps->n_fonts = 0;
   ps->n_fonts_reencoded = 0;
   ps->n_fonts_max = 16;
@@ -790,6 +808,7 @@ gnome_print_ps_new (GnomePrinter *printer)
 				   "%%%% Creator: Gnome Print Version %s\n"
 				   "%%%% DocumentName: %s\n"
 				   "%%%% Author: %s\n"
+				   "%%%%Pages: (atend)\n"
 				   "%%%%EndComments\n\n\n",
 				   VERSION,
 				   "Document Name Goes Here",
