@@ -15,6 +15,7 @@
 #include <libgnomeprint/gnome-print-ps.h>
 #include <libgnomeprint/gnome-print-pixbuf.h>
 #include <string.h>
+#include <locale.h>
 
 static void gnome_print_context_class_init (GnomePrintContextClass *klass);
 
@@ -86,6 +87,7 @@ gnome_print_context_class_init (GnomePrintContextClass *class)
 	class->grayimage = NULL;
 	class->rgbimage = NULL;
 	class->showpage = NULL;
+	class->beginpage = NULL;
 	class->setopacity = NULL;
 }
 
@@ -396,6 +398,15 @@ gnome_print_showpage (GnomePrintContext *pc)
 	return PRINT_CLASS(pc)->showpage (pc);
 }
 
+int
+gnome_print_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
+{
+	g_return_val_if_fail (pc != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+
+	return PRINT_CLASS(pc)->beginpage (pc, name_of_this_page);
+}
+
 int
 gnome_print_setopacity (GnomePrintContext *pc, double opacity)
 {
@@ -496,14 +507,22 @@ gnome_print_context_fprintf (GnomePrintContext *pc, const char *fmt, ...)
 {
 	va_list ap;
 	int n_bytes;
+	char *oldlocale;
 
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (pc->f != NULL, -1);
 
+	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
+	setlocale (LC_NUMERIC, "C");
+
 	va_start (ap, fmt);
 	n_bytes = vfprintf (pc->f, fmt, ap);
 	va_end (ap);
+
+	setlocale (LC_NUMERIC, oldlocale);
+	g_free (oldlocale);
+
 	return n_bytes;
 }
 
