@@ -3174,6 +3174,13 @@ static char *  encode_table_4_names[] = {
   loaded_font->encoding = enc_isolatin;
   loaded_font->id_charstrings = gt1_name_context_intern (psc->nc, "CharStrings");
 
+  gt1_free (enc_std);
+  gt1_free (enc_utf8);
+  gt1_free (encode_table_1);
+  gt1_free (encode_table_2);
+  gt1_free (encode_table_3);
+  gt1_free (encode_table_4);
+  
   return loaded_font;
 }
 
