@@ -3,9 +3,10 @@
 
 /* Header file for parset1.
 
-   You have to include this header additionally:
+   You have to include these headers additionally:
 
    libart_lgpl/art_bpath.h
+   libart_lgpl/art_rect.h
 
 */
 
@@ -13,8 +14,16 @@ typedef struct _Gt1LoadedFont Gt1LoadedFont;
 Gt1LoadedFont * gt1_load_font (const char *filename);
 void gt1_unload_font (Gt1LoadedFont *);
 
-ArtBpath *
-gt1_get_glyph_outline (Gt1LoadedFont *font, int glyphnum, double *p_wx);
+typedef struct {
+	int       ch;
+	ArtBpath *bpath;
+	ArtDRect  bbox;
+	double    wx;
+} Gt1GlyphOutline;
+
+Gt1GlyphOutline *gt1_glyph_outline_lookup (Gt1LoadedFont *, int glyphnum);
+
+ArtBpath *gt1_get_glyph_outline (Gt1LoadedFont *font, int glyphnum, double *p_wx);
 
 double
 gt1_get_kern_pair (Gt1LoadedFont *font, int glyph1, int glyph2);
